# Copyright (C) 2008, OLPC
# Copyright (C) 2010, Walter Bender, Sugar Labs
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import gtk
import gobject
from gettext import gettext as _

from sugar.graphics.icon import Icon
from sugar.graphics import style
from sugar.graphics.xocolor import XoColor

from jarabe.controlpanel.sectionview import SectionView
from jarabe.controlpanel.inlinealert import InlineAlert

_CAPACITIVE = 0
_RESISTIVE = 1

class TouchpadEventIcon(gtk.EventBox):
    """A subclass of the Sugar Event Icon"""
    __gtype_name__ = "SugarEventIcon"

    def __init__(self, **kwargs):
        """Create an extra-large, clickable icon."""    
        gtk.EventBox.__init__(self)

        self.icon = Icon(pixel_size = style.XLARGE_ICON_SIZE, **kwargs)
        
        self.set_visible_window(False)
        self.set_app_paintable(True)
        self.set_events(gtk.gdk.BUTTON_PRESS_MASK)

        self.add(self.icon)
        self.icon.show()

class TouchpadPicker(TouchpadEventIcon):
    """A class for the touchpad selection buttons"""
    __gsignals__ = {
        'touchpad-changed': (gobject.SIGNAL_RUN_FIRST,
                          gobject.TYPE_NONE,
                          ([object]))
    }

    def __init__(self, touchpad_mode):
        """Create icons for the touchpad mode buttons."""    
        TouchpadEventIcon.__init__(self)

        if touchpad_mode == _CAPACITIVE:
            self.icon.props.icon_name = 'capacitive'
        else:
            self.icon.props.icon_name = 'resistive'
        self._touchpad_mode = touchpad_mode

        self.icon.props.pixel_size = style.XLARGE_ICON_SIZE

        self.connect('button_press_event', self.__pressed_cb, touchpad_mode)

    def update(self, touchpad, xo_color):
        """Update the button states."""
        if self._touchpad_mode == _CAPACITIVE:
            self.icon.props.icon_name = 'capacitive'
            if touchpad == _CAPACITIVE:
                self.icon.props.xo_color = xo_color
            else:
                self.icon.props.xo_color = XoColor('#666666,#FFFFFF')
        elif self._touchpad_mode == _RESISTIVE:
            self.icon.props.icon_name = 'resistive'
            if touchpad == _RESISTIVE:
                self.icon.props.xo_color = xo_color
            else:
                self.icon.props.xo_color = XoColor('#666666,#FFFFFF')

    def __pressed_cb(self, button, event, touchpad_mode):
        """Callback for button click."""
        self.emit('touchpad-changed', touchpad_mode)

class Touchpad(SectionView):
    """A class for the touchpad selection panel"""

    def __init__(self, model, alerts):
        """ Create the touchpad panel."""
        SectionView.__init__(self)

        self._model = model
        self._color = XoColor(self._model.get_color_xo())
        self._handlers = []

        self.set_border_width(style.DEFAULT_SPACING * 2)
        self.set_spacing(style.DEFAULT_SPACING)
        self._group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)

        self._touchpad_label = gtk.HBox(spacing=style.DEFAULT_SPACING)
        self._touchpad_box = gtk.HBox(spacing=style.DEFAULT_SPACING)
        self._touchpad_alert_box = gtk.HBox(spacing=style.DEFAULT_SPACING)
        self._touchpad_alert = None

        self._pickers = {
                _CAPACITIVE: TouchpadPicker(_CAPACITIVE),
                _RESISTIVE: TouchpadPicker(_RESISTIVE)}

        self._setup_touchpad()
        self._initial_value = self._model.get_touchpad()
        self._update_pickers(self._initial_value)

        self.setup()

    def _setup_touchpad(self):                
        """Layout the panel: label, buttons, alert"""
        label_touchpad = gtk.Label(_('Click to change your touchpad:'))
        label_touchpad.modify_fg(gtk.STATE_NORMAL, 
                              style.COLOR_SELECTION_GREY.get_gdk_color())
        self._group.add_widget(label_touchpad)
        self._touchpad_label.pack_start(label_touchpad, expand=False)
        label_touchpad.show()

        for touchpad_mode in sorted(self._pickers.keys()):
            picker = self._pickers[touchpad_mode]
            picker.show()
            self._touchpad_box.pack_start(picker, expand=False)

        label_touchpad_error = gtk.Label()
        self._group.add_widget(label_touchpad_error)
        self._touchpad_alert_box.pack_start(label_touchpad_error, expand=False)
        label_touchpad_error.show()

        self._touchpad_alert = InlineAlert()
        self._touchpad_alert_box.pack_start(self._touchpad_alert)

        self._center_in_panel = gtk.Alignment(0.5)
        self._center_in_panel.add(self._touchpad_box)
        self.pack_start(self._touchpad_label, False)
        self.pack_start(self._center_in_panel, False)
        self.pack_start(self._touchpad_alert_box, False)        
        self._touchpad_label.show()
        self._touchpad_box.show()
        self._touchpad_alert_box.show()
        self._center_in_panel.show()
    
    def setup(self):
        """Set up the buttons."""
        self.needs_restart = False

        def connect(widget, signal, callback):
            self._handlers.append((widget, widget.connect(signal, callback)))

        for picker in self._pickers.values():
            connect(picker, 'touchpad-changed', self.__touchpad_changed_cb)

    def undo(self):
        """Undo any changes."""
        for widget, handler in self._handlers:
            widget.disconnect(handler)
        self._model.set_touchpad(self._initial_value)
        self._touchpad_alert.hide()        

    def _update_pickers(self, touchpad):
        """Update the buttons to reflect selection"""
        for picker in self._pickers.values():
            picker.update(touchpad, self._color)

    def __touchpad_changed_cb(self, touchpadpicker, touchpad):
        """Callback for mode change -- no restart alert is needed."""
        self._model.set_touchpad(touchpad)
        self._update_pickers(touchpad)
        return False
